
import unittest

from nanohttp import html, RestController
from nanohttp.tests.helpers import WsgiAppTestCase


class RestDispatcherTestCase(WsgiAppTestCase):

    class Root(RestController):

        @html
        def get(self, article_id: int = None):
            yield "GET Article%s" % (
                's' if not article_id else (': ' + article_id)
            )

        @html
        def post(self):
            yield "POST Article"

        @html
        def put(self, article_id: int = None):
            yield "PUT Article: %s" % article_id

        def disallowed(self):  # pragma: nocover
            yield 'bad'

    def test_verbs(self):
        self.assert_get('/', expected_response='GET Articles')
        self.assert_post('/', expected_response='POST Article')
        self.assert_put('/12', expected_response='PUT Article: 12')
        self.assert_get('/disallowed', status=404)


if __name__ == '__main__':  # pragma: no cover
    unittest.main()



import sys
import unittest
import base64
import mimetypes
import socket
import io
import traceback
from hashlib import md5
from os import path, urandom
from urllib.parse import urlencode

import httplib2
from wsgi_intercept import WSGIAppError
from wsgi_intercept.interceptor import Httplib2Interceptor

from nanohttp import Controller, configure, Application

TEST_DIR = path.abspath(path.dirname(__file__))
STUFF_DIR = path.join(TEST_DIR, 'stuff')


def md5sum(f):
    if isinstance(f, str):
        file_obj = open(f, 'rb')
    else:
        file_obj = f

    try:
        checksum = md5()
        while True:
            d = file_obj.read(0x8000)
            if not d:
                break
            checksum.update(d)
        return checksum.digest()
    finally:
        if file_obj is not f:
            file_obj.close()


class WsgiTester(httplib2.Http):

    def __init__(self, app_factory, host='nanohttp.org', port=80, **kw):
        super(WsgiTester, self).__init__()
        self.interceptor = Httplib2Interceptor(
            app_factory,
            host=host,
            port=port,
            **kw
        )

    def __enter__(self):
        self.interceptor.__enter__()
        return self

    def __exit__(self, exc_type, value, traceback):
        self.interceptor.__exit__(exc_type, value, traceback)

    def request(self, uri, query_string=None, fields=None, files=None,
                json=None, **kw):
        headers = kw.setdefault('headers', {})
        body = None

        if files:
            content_type, body, length = encode_multipart_data(fields, files)
            body = body.getvalue()
            headers.setdefault('Content-Type', content_type)
        elif fields:
            body = urlencode(fields)
            headers.setdefault(
                'Content-Type',
                'application/x-www-form-urlencoded'
            )
        elif json:
            body = json
            headers.setdefault('Content-Type', 'application/json')

        if query_string:
            uri +=\
                '%s%s' % ('&' if '?' in uri else '?', urlencode(query_string))

        try:
            return super(WsgiTester, self).request(
                '%s%s' % (self.interceptor.url, uri),
                body=body,
                **kw
            )
        except WSGIAppError as e:
            content = traceback.format_exc().encode()

            # Closing the dirty(excepted) connection to force use a fresh one
            # for the next request. see:
            # https://github.com/Carrene/nanohttp/issues/67
            for c in (self.connections or {}).values():
                c.close()

            # Create response object to indicate a 500
            response = httplib2.Response({
                "content-type": "text/plain",
                "status": "500",
                "content-length": len(content)
            })
            response.reason = content
            #response.status = 500
            return response, content


class WsgiAppTestCase(unittest.TestCase):

    class Root(Controller):
        pass

    class Application(Application):
        pass

    def setUp(self):
        configure(force=True)
        self.application = self.Application(self.Root())
        self.client = WsgiTester(lambda: self.application)
        self.client.__enter__()

    def tearDown(self):
        self.client.__exit__(*sys.exc_info())

    def assert_request(self, uri, method, cookies=None, expected_response=None,
                       expected_checksum=None,not_expected_headers=None,
                       expected_headers=None, status=200, **kw):

        if cookies:
            headers = kw.setdefault('headers', {})
            headers['Cookie'] = '; '.join(
                '%s=%s' % (k, v.value) for k, v in cookies.items()
            )

        response, content = self.client.request(uri, method=method, **kw)

        if response.status == 500:
            print(content.decode(), file=sys.stderr)
        self.assertEqual(status, response.status)

        if expected_response is not None:
            if isinstance(expected_response, str):
                self.assertEqual(expected_response, content.decode())
            else:
                self.assertRegex(content.decode(), expected_response)

        for k, v in (expected_headers or {}).items():
            k = k.lower()
            if isinstance(v, str):
                self.assertEqual(response[k], v)
            else:
                self.assertRegex(response[k], v)

        for k in not_expected_headers or []:
            self.assertNotIn(k.lower(), response)

        if expected_checksum:
            self.assertEqual(md5sum(io.BytesIO(content)), expected_checksum)

        return response, content

    def assert_get(self, uri, *args, **kw):
        return self.assert_request(uri, 'get', *args, **kw)

    def assert_options(self, uri, *args, **kw):
        return self.assert_request(uri, 'options', *args, **kw)

    def assert_post(self, uri, *args, **kw):
        return self.assert_request(uri, 'post', *args, **kw)

    def assert_put(self, uri, *args, **kw):
        return self.assert_request(uri, 'put', *args, **kw)

    def assert_delete(self, uri, *args, **kw):
        return self.assert_request(uri, 'delete', *args, **kw)


def encode_multipart_data(fields=None, files=None):  # pragma: no cover
    boundary = ''.join(
        ['-----', base64.urlsafe_b64encode(urandom(27)).decode()]
    )
    crlf = b'\r\n'
    lines = []

    if fields:
        for key, value in fields.items():
            lines.append('--' + boundary)
            lines.append('Content-Disposition: form-data; name="%s"' % key)
            lines.append('')
            lines.append(value if isinstance(value, str) else str(value))

    if files:
        for key, file_path in files.items():
            filename = path.split(file_path)[1]
            lines.append('--' + boundary)
            lines.append(
                'Content-Disposition: form-data; name="%s"; filename="%s"' %
                (key, filename))
            lines.append(
                'Content-Type: %s' %
                (
                    mimetypes.guess_type(filename)[0] or
                    'application/octet-stream')
            )
            lines.append('')
            with open(file_path, 'rb') as f:
                lines.append(f.read())

    lines.append('--' + boundary + '--')
    lines.append('')

    body = io.BytesIO()
    length = 0
    for l in lines:
        # noinspection PyTypeChecker
        line = (l if isinstance(l, bytes) else l.encode()) + crlf
        length += len(line)
        body.write(line)
    body.seek(0)
    content_type = 'multipart/form-data; boundary=%s' % boundary
    return content_type, body, length


def find_free_tcp_port():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.bind((socket.gethostname(), 0))
        return s.getsockname()[1]
    finally:
        s.close()


if __name__ == '__main__':  # pragma: no cover
    ct, b, len_ = encode_multipart_data(
        dict(
            test1='TEST1VALUE'
        ),
        files=dict(
            cat='stuff/cat.jpg'
        )
    )
    print(ct)
    print(len_)
    print(b.read())



import unittest

from nanohttp.helpers import LazyAttribute


class LazyAttributeTestCase(unittest.TestCase):
    def test_wrapping(self):

        class O:
            @LazyAttribute
            def attr1(self):
                """Attribute 1"""
                return 1

        self.assertEqual('Attribute 1', O.attr1.__doc__)
        self.assertEqual('attr1', O.attr1.__name__)

        o = O()
        self.assertEqual(1, o.attr1)


if __name__ == '__main__':  # pragma: no cover
    unittest.main()




import re
import unittest
from decimal import Decimal

from nanohttp import HTTPBadRequest, HTTPStatus, Controller, text, validate, \
    context
from nanohttp.tests.helpers import WsgiAppTestCase
from nanohttp.validation import RequestValidator


class ValidationTestCase(unittest.TestCase):

    def test_validation_required(self):

        validator = RequestValidator(
            fields=dict(
                param1=dict(required=True)
            )
        )

        # Trying to pass with param1
        self.assertEqual(
            (dict(param1='value1'), None),
            validator(dict(param1='value1'))
        )

        # Trying to pass without param1
        with self.assertRaises(HTTPBadRequest):
            validator(dict(another_param='value1'))

        # Define required validation with custom exception
        validator = RequestValidator(
            fields=dict(
                param1=dict(required='600 Custom exception')
            )
        )

        # Trying to pass with another param without param1
        with self.assertRaises(HTTPStatus('600 Custom exception').__class__):
            validator(dict(another_param='value1'))

        # Trying to pass with param1
        self.assertEqual(
            (dict(param1='value1'), None),
            validator(dict(param1='value1'))
        )

    def test_validation_max(self):

        validator = RequestValidator(
            fields=dict(
                param1=dict(maximum=10)
            )
        )
        self.assertEqual((dict(param1=9), None), validator(dict(param1=9)))
        self.assertEqual(
            (dict(param1=None), None),
            validator(dict(param1=None))
        )
        with self.assertRaises(HTTPBadRequest):
            self.assertEqual(
                (dict(param1=9), None),
                validator(dict(param1='9'))
            )

        # More than Expectation
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1=11))

        # Not int input
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1='a'))

    def test_validation_min(self):

        validator = RequestValidator(
            fields=dict(
                param1=dict(minimum=10)
            )
        )

        self.assertEqual((dict(param1=11), None), validator(dict(param1=11)))
        with self.assertRaises(HTTPBadRequest):
            self.assertEqual(
                (dict(param1=11), None),
                validator(dict(param1='11'))
            )

        # Less than Expectation
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1=9))

    def test_validation_min_length(self):

        validator = RequestValidator(
            fields=dict(
                param1=dict(min_length=3)
            )
        )
        self.assertEqual(
            (dict(param1='abcd'), None),
            validator(dict(param1='abcd'))
        )
        self.assertEqual(
            (dict(param1='abc'), None),
            validator(dict(param1='abc'))
        )
        self.assertEqual(
            (dict(param1='1234'), None),
            validator(dict(param1='1234'))
        )

        # Shorter than Expectation
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1='ab'))

    def test_validation_max_length(self):
        validator = RequestValidator(
            fields=dict(
                param1=dict(max_length=4)
            )
        )
        self.assertEqual(
            (dict(param1='abc'), None),
            validator(dict(param1='abc'))
        )
        self.assertEqual(
            (dict(param1='abcd'), None),
            validator(dict(param1='abcd'))
        )
        self.assertEqual(
            (dict(param1='1234'), None),
            validator(dict(param1='1234'))
        )

        # Longer than Expectation
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1='abbcde'))

    def test_validation_pattern(self):
        validator = RequestValidator(
            fields=dict(
                param1=dict(pattern='^\D{10}$')
            )
        )

        self.assertEqual(
            (dict(param1='abcdeFGHIJ'), None),
            validator(dict(param1='abcdeFGHIJ'))
        )

        # Param1 not matching
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1='asc'))

    def test_validation_pattern_compiled(self):
        validator = RequestValidator(
            fields=dict(
                param1=dict(pattern=re.compile(r'^\d{10}$'))
            )
        )

        self.assertEqual(
            (dict(param1='0123456789'), None),
            validator(dict(param1='0123456789'))
        )

        # Param1 not matching
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1='12345'))

    def test_validation_type(self):
        validator = RequestValidator(
            fields=dict(
                param1=dict(type_=int),
                param2=dict(type_=str),
                param3=dict(type_=Decimal)
            )
        )

        self.assertEqual(
            (dict(param1=123, param2='123', param3=1.23), None),
            validator(dict(param1='123', param2=123, param3=1.23))
        )

        # Param1 bad value(Cannot be converted to int)
        with self.assertRaises(HTTPBadRequest):
            validator(
                dict(
                    param1='NotInteger',
                    param2='NotInteger',
                    param3='NotInteger'
                )
            )

        # Param3 bad value(Cannot be converted to decimal)
        with self.assertRaises(HTTPBadRequest):
            validator(
                dict(param1=1, param2='str', param3='NotDecimal'))

    def test_validation_query_string(self):

        # Accept query_string
        validator = RequestValidator(
            fields=dict(
                param1=dict(query=True),
            )
        )

        self.assertEqual(
            (None, dict(param1='value')),
            validator(query=dict(param1='value'))
        )

    def test_validation_custom_status(self):
        validator = RequestValidator(
            fields=dict(
                param1=dict(
                    type_=(int, '999 Type error'),
                    minimum=(30, '666'),
                    maximum=(40, '400 greater than maximum'),
                )
            )
        )

        try:
            validator(dict(param1='NotInteger'))
        except HTTPStatus as e:
            self.assertEqual(e.status, '999 Type error')

        try:
            validator(dict(param1=29))
        except HTTPStatus as e:
            self.assertEqual(e.status, '666 Bad request')

        try:
            validator(dict(param1=41))
        except HTTPStatus as e:
            self.assertEqual(e.status, '400 greater than maximum')

    def test_callable_validator(self):
        def f(age, container, field):
            age = int(age)
            if age < 0:
                raise ValueError('Age is not in valid range')
            return age

        validator = RequestValidator(
            fields=dict(
                param1=f
            )
        )

        with self.assertRaises(ValueError):
            validator(dict(param1=-1))

        self.assertEqual(
            dict(param1=12).items(),
            validator(dict(param1=12))[0].items()
        )

    def test_not_none_validator(self):
        validator = RequestValidator(fields=dict(param1=dict(not_none=True)))
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1=None))

        validator = RequestValidator(
            fields=dict(param1=dict(not_none='666 param1 is null'))
        )
        with self.assertRaises(HTTPStatus) as ctx:
            validator(dict(param1=None))
        exception = ctx.exception
        self.assertEqual('666 param1 is null', str(exception))

        with self.assertRaises(TypeError):
            RequestValidator(fields=dict(param1=dict(not_none=23)))

    def test_readonly_validator(self):
        validator = RequestValidator(fields=dict(param1=dict(readonly=True)))
        with self.assertRaises(HTTPBadRequest):
            validator(dict(param1=None))

        validator = RequestValidator(
            fields=dict(param1=dict(not_none='666 param1 is readonly'))
        )
        with self.assertRaises(HTTPStatus) as ctx:
            validator(dict(param1=None))
        exception = ctx.exception
        self.assertEqual('666 param1 is readonly', str(exception))


class ValidationDecoratorTestCase(WsgiAppTestCase):
    class Root(Controller):

        @validate(
            query1=dict(
                required=True,
                type_=int,
                query=True
            ),
            field1=dict(
                required=True,
                type_=float
            ),
            field2=dict(
                required=True,
                type_=int
            ),
            field3=dict(
                required=True,
                type_=lambda v: v.encode(),
                min_length=1
            )
        )
        @text(methods='post')
        def index(self):
            query1 = context.query['query1']
            field1 = context.form['field1']
            field2 = context.form['field2']
            field3 = context.form['field3']
            yield \
                f'{type(query1).__name__}: {query1}, '\
                f'{type(field1).__name__}: {field1}, ' \
                f'{type(field2).__name__}: {field2}, ' \
                f'{type(field3).__name__}: {field3}'

    def test_validation_decorator(self):
        response, body = self.assert_post(
            '/?query1=1',
            fields=dict(field1='2.3', field2='2', field3='ab'),
            expected_response='int: 1, float: 2.3, int: 2, bytes: b\'ab\''
        )


if __name__ == '__main__':  # pragma: no cover
    unittest.main()



import unittest

from nanohttp import Controller, action, text, HTTPMovedPermanently, HTTPFound
from nanohttp.tests.helpers import WsgiAppTestCase


class HTTPRedirectTestCase(WsgiAppTestCase):

    class Root(Controller):

        @text()
        def index(self):
            raise HTTPMovedPermanently('/new/address')

        @action()
        def about(self):
            raise HTTPFound('/new/address')

    def test_redirect_response_header(self):
        self.assert_get(
            '/',
            status=301,
            expected_headers={'Location': '/new/address'}
        )
        self.assert_get(
            '/about',
            status=302,
            expected_headers={'Location': '/new/address'}
        )


if __name__ == '__main__':  # pragma: no cover
    unittest.main()




import unittest

from nanohttp import Controller, html, json, action, RestController, text,\
    RegexRouteController
from nanohttp.tests.helpers import WsgiAppTestCase


class LinksController(Controller):

    @html
    def index(self, article_id: int=None, link_id: int=None):
        if link_id:
            yield 'Article: %s, Links index: %s' % (article_id, link_id)
        else:
            yield 'Link1, Link2, Link3'


class ArticleController(RestController):
    links = LinksController()

    @html
    def get(self, article_id: int=None):
        yield 'GET Article%s' % (
            's' if not article_id else (': ' + article_id)
        )

    @html
    def post(self):
        yield 'POST Article'

    @html
    def put(self, article_id: int=None, inner_resource: str=None,
            link_id: int=None):
        if inner_resource == 'links':
            yield from self.links(article_id, link_id)
        else:
            yield 'PUT Article: %s' % article_id

    def disallowed(self):  # pragma: no cover
        yield 'bad'

    @text
    def remove(self, id_: int):
        yield 'Removing, %s' % id_


class AwesomeRegexController(RegexRouteController):

    def __init__(self):
        super().__init__([
            ('/awesome/(?P<name>\d+)', self.awesome_action)
        ])

    @text
    def awesome_action(self, name):
        return name


class NoIndexController(Controller):
    pass


class IndexParamController(Controller):

    @text
    def index(self, *args):
        yield '/'.join(args)


class DispatcherTestCase(WsgiAppTestCase):

    class Root(Controller):
        no_index = NoIndexController()
        articles = ArticleController()
        regex = AwesomeRegexController()
        params = IndexParamController()

        @html
        def index(self):
            yield 'Index'

        @json(verbs='post')
        def login(self):
            return ['token']

        @action
        def tuple(self):
            return ('the', 'tuple')

        @html(verbs=['get', 'post'])
        def contact(self, contact_id: int=None):
            yield 'Contact: %s' % contact_id

        @html
        def users(self, user_id: int=None, attr: str=None):
            yield 'User: %s\n' % user_id
            yield 'Attr: %s\n' % attr

        @html
        def books(self, name=None, *, sort='user_id', filters=None):
            yield 'name: %s sort: %s filters: %s' % (name, sort, filters)

        @html
        def bad(self):
            raise Exception()

        @text
        def empty(self):
            pass

    def test_root(self):
        self.assert_get('/', expected_response='Index')
        self.assert_get('/index', expected_response='Index')

    def test_arguments(self):
        self.assert_get('/contact/1', expected_response='Contact: 1')
        self.assert_post('/contact', expected_response='Contact: None')
        self.assert_get('/users', expected_response='User: None\nAttr: None\n')
        self.assert_get('/users/10/')
        self.assert_get(
            '/users/10/jobs',
            expected_response='User: 10\nAttr: jobs\n'
        )
        self.assert_get('/users/10/11/11', status=404)
        self.assert_post('/articles/2', status=404)
        self.assert_get('/tuple')

    def test_verbs(self):
        self.assert_put('/', expected_response='Index')
        self.assert_post('/login', expected_response='[\n    "token"\n]')
        self.assert_get('/contact/test', expected_response='Contact: test')
        self.assert_post('/contact', expected_response='Contact: None')
        self.assert_put('/contact', status=405)
        self.assert_options('/contact', status=405)
        self.assert_options('/')
        self.assert_delete('/articles', status=405)
        self.assert_get('/no_index', status=404)

    def test_rest_controller(self):
        self.assert_get('/articles', expected_response='GET Articles')
        self.assert_get('/articles/23', expected_response='GET Article: 23')
        self.assert_post('/articles', expected_response='POST Article')
        self.assert_put('/articles/23', expected_response='PUT Article: 23')
        self.assert_put(
            '/articles/23/links/2',
            expected_response='Article: 23, Links index: 2'
        )
        self.assert_get(
            '/articles/links',
            expected_response='Link1, Link2, Link3'
        )
        self.assert_request('/articles', 'disallowed', status=404)
        self.assert_get('/articles/disallowed', status=404)

    def test_trailing_slash(self):
        self.assert_get(
            '/users/10/jobs/',
            expected_response='User: 10\nAttr: jobs\n'
        )

    def test_exception(self):
        self.assert_get('/bad', status=500)

    def test_empty_response(self):
        self.assert_get('/empty', expected_response='')

    def test_positional_argument_dispatch(self):
        self.assert_request(
            '/articles/23',
            'remove',
            expected_response='Removing, 23'
        )

    def test_dispatch_query_string(self):
        self.assert_get(
            '/books',
            expected_response='name: None sort: user_id filters: None'
        )
        self.assert_get(
            '/books/C++',
            expected_response='name: C++ sort: user_id filters: None'
        )
        self.assert_get(
            '/books?sort=title',
            expected_response='name: None sort: title filters: None'
        )
        self.assert_get(
            '/books/?filters=a>1&filters=b<3',
            expected_response=\
                'name: None sort: user_id filters: '
                '[\'a>1\', \'b<3\']'
        )

    def test_regex_route_controller(self):
        self.assert_get('/regex/awesome/badinteger', status=404)
        self.assert_get('/regex/awesome/123', expected_response='123')

    def test_index_parameters(self):
        self.assert_get('/params/')


if __name__ == '__main__':  # pragma: no cover
    unittest.main()



import unittest
import re

from nanohttp import Controller, action, text, html, context, chunked
from nanohttp.contexts import Context
from nanohttp.tests.helpers import WsgiAppTestCase


class ChunkedEncodingTestCase(WsgiAppTestCase):


    @staticmethod
    @chunked
    def chunked_action():
        yield 'first chunk'
        yield 'second chunk'
        yield 'third chunk'

    @staticmethod
    @chunked('field', 'value')
    def trailer_action():
        yield 'first chunk'
        yield 'second chunk'

    @staticmethod
    @chunked('field', 'value')
    def error_action():
        yield 'first chunk'
        yield 1/0

    def test_chunked(self):
        output_generator = self.chunked_action()
        with Context(environ={}) as context:
            self.assertEqual(next(output_generator), '11\r\nfirst chunk\r\n')
            self.assertEqual(next(output_generator), '12\r\nsecond chunk\r\n')
            self.assertEqual(next(output_generator), '11\r\nthird chunk\r\n')
            self.assertEqual(next(output_generator), '0\r\n')
            self.assertEqual(next(output_generator), '\r\n')

            self.assertEqual(
                context.response_headers['transfer-encoding'], 'chunked'
            )

    def test_trailer(self):
        output_generator = self.trailer_action()
        with Context(environ={}) as context:
            self.assertEqual(next(output_generator), '11\r\nfirst chunk\r\n')
            self.assertEqual(next(output_generator), '12\r\nsecond chunk\r\n')
            self.assertEqual(next(output_generator), '0\r\n')
            self.assertEqual(next(output_generator), 'field: value\r\n')
            self.assertEqual(next(output_generator), '\r\n')
            self.assertEqual(context.response_headers['trailer'], 'field')

    def test_error(self):
        output_generator = self.error_action()
        with Context(environ={}) as context:
            self.assertEqual(next(output_generator), '11\r\nfirst chunk\r\n')
            self.assertEqual(next(output_generator), 'division by zero')
            self.assertEqual(next(output_generator), '0\r\n')
            self.assertEqual(next(output_generator), '\r\n')


if __name__ == '__main__':  # pragma: no cover
    unittest.main()




import threading
import time
import unittest
from os import path

import httplib2

from nanohttp import main, settings
from nanohttp.tests.helpers import find_free_tcp_port


here = path.abspath(path.dirname(__file__))


class EntryPointTestCase(unittest.TestCase):

    def setUp(self):
        self.demo_filename = path.join(here, 'stuff/demo.txt')
        self.module_dir = path.abspath(path.join(here, '..'))
        with open(self.demo_filename, mode='w') as f:
            f.write('some text')
        self.port = find_free_tcp_port()
        self.url = f'http://localhost:{self.port}/tests/stuff/demo.txt'

    def test_main_function(self):
        args = [
            'nanohttp',
            f'-C', self.module_dir,
            f'-b', str(self.port),
            ':Static',
        ]

        t = threading.Thread(target=main, args=(args,), daemon=True)
        t.start()

        time.sleep(1)

        client = httplib2.Http()
        response, content = client.request(self.url)
        self.assertIsNotNone(response)
        self.assertIsNotNone(content)

    def test_configuration_file(self):

        filename = path.join(here, 'stuff/sample.yml')

        args = [
            'nanohttp',
            f'-C', self.module_dir,
            f'--bind', str(self.port),
            f'--config-file', filename,
            ':Static',
        ]
        t = threading.Thread(target=main, args=(args,), daemon=True)
        t.start()

        time.sleep(.2)
        self.assertEqual('value', settings.custom_key)

    def test_configuration_cli_option_o_flag(self):

        filename = path.join(here, 'stuff/sample.yml')

        args = [
            'nanohttp',
            f'-C', self.module_dir,
            f'--bind', str(self.port),
            f'--config-file', filename,
            f'--option', 'custom_key=hello',
            ':Static',
        ]
        t = threading.Thread(target=main, args=(args,), daemon=True)
        t.start()

        time.sleep(.5)
        self.assertEqual('hello', settings.custom_key)

    def test_invalid_configuration_cli_option_o_flag(self):

        filename = path.join(here, 'stuff/sample.yml')

        args = [
            'nanohttp',
            f'-C', self.module_dir,
            f'--bind', str(self.port),
            f'--config-file', filename,
            f'--option', 'invalid.key=hello',
            ':Static',
        ]
        t = threading.Thread(target=main, args=(args,), daemon=True)
        t.start()

        time.sleep(.5)


if __name__ == '__main__':  # pragma: no cover
    unittest.main()




import unittest

from nanohttp import Controller, action, context, RestController
from nanohttp.tests.helpers import WsgiAppTestCase


class Users(RestController):
    @action
    def post(self):
        return ', '.join('%s: %s' % (k, v)
            for k, v in sorted(context.query.items(), key=lambda x: x[0]))


class QueryStringTestCase(WsgiAppTestCase):

    class Root(Controller):
        users = Users()

        @action()
        def index(self):
            return ', '.join('%s: %s' % (k, v)
                for k, v in sorted(context.query.items(), key=lambda x: x[0]))

    def test_simple_query_string(self):
        self.assert_get('/?a=1&b=&c=2', expected_response='a: 1, b: , c: 2')
        self.assert_get('/?a=1&b=2', expected_response='a: 1, b: 2')
        self.assert_get(
            '/?a=1&b=2&b=3',
            expected_response='a: 1, b: [\'2\', \'3\']'
        )
        self.assert_get(
            '/',
            query_string=dict(a=1, b=2),
            expected_response='a: 1, b: 2'
        )
        self.assert_post(
            '/',
            query_string=dict(a=1, b=2),
            expected_response='a: 1, b: 2'
        )

    def test_rest_controller_query_string(self):
        self.assert_post(
            '/users',
            query_string=dict(a=1, b=2),
            expected_response='a: 1, b: 2'
        )

    def test_escaping(self):
        self.assert_get('/?a=%26%20%21&b=2', expected_response='a: & !, b: 2')


if __name__ == '__main__':  # pragma: no cover
    unittest.main()




import unittest

import ujson

from nanohttp import Controller, html, HTTPBadRequest, json, HTTPStatus, \
    settings, HTTPNotModified
from nanohttp.tests.helpers import WsgiAppTestCase


class ExceptionTestCase(WsgiAppTestCase):

    class Root(Controller):
        @html
        def index(self):
            raise HTTPBadRequest()

        @json
        def data(self):
            raise HTTPBadRequest()

        @json
        def custom(self):
            raise HTTPStatus(status='462 custom text')

        @html
        def err(self):
            return 1 / 0

        @html
        def not_modified(self):
            raise HTTPNotModified()

    def test_exception(self):
        response, content = self.assert_get('/', status=400)
        self.assertIsNotNone(content)

        response, content = self.assert_get('/data', status=400)
        self.assertIn('stackTrace', ujson.loads(content))
        self.assertIsNotNone('stackTrace', ujson.loads(content))

        response, content = self.assert_get('/err', status=500)
        self.assertIsNotNone(content)

        settings.debug = False

        response, content = self.assert_get('/', status=400)
        self.assertEqual(content, b'')

        response, content = self.assert_get('/data', status=400)
        self.assertNotIn('stackTrace', ujson.loads(content))

        self.assert_get('/not_modified', status=304)

    def test_custom_exception(self):
        response, content = self.assert_get('/custom', status=462)

        self.assertIn('stackTrace', ujson.loads(content))
        self.assertIsNotNone('stackTrace', ujson.loads(content))


if __name__ == '__main__':  # pragma: no cover
    unittest.main()
